<?php
/**
 * Theme Update Checker Library 1.0
 *
 */

if ( !class_exists('ImwbThemeUpdateChecker_nolic') ) {

/**
 * A custom theme update checker
 *
 */
class ImwbThemeUpdateChecker_nolic {
	public $apiKey = '';                    //The API Key to enable access to the update and license server
	public $themeSlug = '';                 //The theme associated with this update checker instance.
	public $metadataUrl = '';               //The URL of the theme's metadata file.
	public $enableAutomaticChecking = true; //Enable/disable automatic update checks.
	public $enableLicenseCheck = false;     //Enable/disable automatic update checks.

	protected $optionName = '';      //Where to store update info.
	protected $automaticCheckDone = false;
	protected static $filterPrefix = 'imwb_theme_update_';
   protected static $updateServerUrl = 'http://digitalguard.it/wp-update-server/nolic.php';

	/**
	 * Class constructor.
	 *
	 * @param string $themeSlug Theme slug, e.g. "twentyten".
	 * @param string $optionNamePrefix The prefix for the option name.
	 * @param boolean $enableAutomaticChecking Enable/disable automatic update checking. If set to FALSE, you'll need to explicitly call checkForUpdates() to, err, check for updates.
	 * @param boolean $enableLicenseCheck Enable/Disable license checking, if enabled must hook the action ??????? to supply the license code that is used in conjunction with the Imwb Activation system.
	 */
	public function __construct($apiKey, $themeSlug, $optionNamePrefix, $enableAutomaticChecking = true, $enableLicenseCheck = false){

	  $this->apiKey = $apiKey;
		$this->metadataUrl = self::$updateServerUrl.'?action=get_metadata&slug='.$themeSlug;
		$this->enableAutomaticChecking = $enableAutomaticChecking;
		$this->enableLicenseCheck = $enableLicenseCheck;
		$this->themeSlug = $themeSlug;
		$this->optionName = $optionNamePrefix.'-'.$this->themeSlug;

		$this->installHooks();
	}

	/**
	 * Install the hooks required to run regular update checks and inject update info
	 * into WP data structures.
	 *
	 * @return void
	 */
	public function installHooks(){
		// Check for updates when WordPress does.
		if ( $this->enableAutomaticChecking )
			add_filter('pre_set_site_transient_update_themes', array($this, 'onTransientUpdateTheme'));

		// Insert our update info into the update list maintained by WP.
		add_filter('site_transient_update_themes', array($this,'injectThemeUpdate'));

		//Delete our update info when WP deletes its own.
		//This usually happens when a theme is installed, removed or upgraded.
		add_action('delete_site_transient_update_themes', array($this, 'deleteStoredData'));
	}

	/**
	 * Retrieve update info from the configured metadata URL.
	 *
	 * Returns either an instance of ImwbThemeUpdate, or NULL if there is
	 * no newer version available or if there's an error.
	 *
	 * @uses wp_remote_get()
	 *
	 * @param array $queryArgs Additional query arguments to append to the request. Optional.
	 * @return ImwbThemeUpdate
	 */
	public function requestUpdate($queryArgs = array()){
		//Query args to append to the URL.
		$queryArgs['api_key'] = $this->apiKey;
		$queryArgs['installed_version'] = $this->getInstalledVersion();
		if ( $this->enableLicenseCheck )
		   $queryArgs = apply_filters(self::$filterPrefix.'query_args-'.$this->themeSlug, $queryArgs);
		//Various options for the wp_remote_get() call. Themes can filter these, too.
		$options = array(
			'timeout' => 10, //seconds
		);
		$options = apply_filters(self::$filterPrefix.'options-'.$this->themeSlug, $options);

		$url = $this->metadataUrl;
		if ( !empty($queryArgs) ){
			$url = add_query_arg($queryArgs, $url);
		}

		//Send the request.
		$result = wp_remote_get($url, $options);

		//Try to parse the response
		$ImwbThemeUpdate = null;
		$code = wp_remote_retrieve_response_code($result);
		$body = wp_remote_retrieve_body($result);
		if ( ($code == 200) && !empty($body) ){
			$ImwbThemeUpdate = ImwbThemeUpdate_nolic::fromJson($body);
			//The update should be newer than the currently installed version.
			if ( ($ImwbThemeUpdate != null) && version_compare($ImwbThemeUpdate->version, $this->getInstalledVersion(), '<=') ){
				$ImwbThemeUpdate = null;
			}
		}

		$ImwbThemeUpdate = apply_filters(self::$filterPrefix.'result-'.$this->themeSlug, $ImwbThemeUpdate, $result);
		return $ImwbThemeUpdate;
	}

	/**
	 * Get the currently installed version of our theme.
	 *
	 * @return string Version number.
	 */
	public function getInstalledVersion(){
		if ( function_exists('wp_get_theme') ) {
			$theme = wp_get_theme($this->themeSlug);
			return $theme->get('Version');
		}

		/** @noinspection PhpDeprecationInspection get_themes() used for compatibility with WP 3.3 and below. */
		foreach(get_themes() as $theme){
			if ( $theme['Stylesheet'] === $this->themeSlug ){
				return $theme['Version'];
			}
		}
		return '';
	}

	/**
	 * Check for theme updates.
	 *
	 * @return void
	 */
	public function checkForUpdates(){
		$state = get_option($this->optionName);
		if ( empty($state) ){
			$state = new StdClass;
			$state->lastCheck = 0;
			$state->checkedVersion = '';
			$state->update = null;
		}

		$state->lastCheck = time();
		$state->checkedVersion = $this->getInstalledVersion();
		update_option($this->optionName, $state); //Save before checking in case something goes wrong

		$state->update = $this->requestUpdate();
		update_option($this->optionName, $state);
	}

	/**
	 * Run the automatic update check, but no more than once per page load.
	 * This is a callback for WP hooks. Do not call it directly.
	 *
	 * @param mixed $value
	 * @return mixed
	 */
	public function onTransientUpdateTheme($value){
		if ( !$this->automaticCheckDone ){
			$this->checkForUpdates();
			$this->automaticCheckDone = true;
		}
		return $value;
	}

	/**
	 * Insert the latest them update (if any) into the update list maintained by WP.
	 *
	 * @param StdClass $updates Update list.
	 * @return array Modified update list.
	 */
	public function injectThemeUpdate($updates){
		$state = get_option($this->optionName);

		//Is there an update to insert?
		if ( !empty($state) && isset($state->update) && !empty($state->update) ){
			$updates->response[$this->themeSlug] = $state->update->toWpFormat();
		}

		return $updates;
	}

	/**
	 * Delete any stored book-keeping data.
	 *
	 * @return void
	 */
	public function deleteStoredData(){
		delete_option($this->optionName);
	}

	/**
	 * Register a callback for filtering query arguments. This must be used when the enableLicenseCheck
	 * is used
	 *
	 * The callback function should take one argument - an associative array of query arguments.
	 * It should return a modified array of query arguments.
	 *
	 * If enableLicenseCheck used then must add a query arg called "license_key" set to the value to check.
	 *
	 * @param callable $callback
	 * @return void
	 */
	public function addQueryArgFilter($callback){
		add_filter(self::$filterPrefix.'query_args-'.$this->themeSlug, $callback);
	}
}

}

if ( !class_exists('ImwbThemeUpdate_nolic') ) {

/**
 * A simple container class for holding information about an available update.
 *
 */
class ImwbThemeUpdate_nolic {
	public $version;      //Version number.
	public $details_url;  //The URL where the user can learn more about this version.
	public $download_url; //The download URL for this version of the theme. Optional.

	/**
	 * Create a new instance of ImwbThemeUpdate_nolic from its JSON-encoded representation.
	 *
	 * @param string $json Valid JSON string representing a theme information object.
	 * @return ImwbThemeUpdate_nolic New instance of ImwbThemeUpdate_nolic, or NULL on error.
	 */
	public static function fromJson($json){
		$apiResponse = json_decode($json);
		if ( empty($apiResponse) || !is_object($apiResponse) ){
			return null;
		}

		//Very, very basic validation.
		$valid = isset($apiResponse->version) && !empty($apiResponse->version) && isset($apiResponse->details_url) && !empty($apiResponse->details_url);
		if ( !$valid ){
			return null;
		}

		$update = new self();
		foreach(get_object_vars($apiResponse) as $key => $value){
			$update->$key = $value;
		}

		return $update;
	}

	/**
	 * Transform the update into the format expected by the WordPress core.
	 *
	 * @return array
	 */
	public function toWpFormat(){
		$update = array(
			'new_version' => $this->version,
			'url' => $this->details_url,
		);

		if ( !empty($this->download_url) ){
			$update['package'] = $this->download_url;
		}

		return $update;
	}
}

}

?>
